@@ -16,7 +16,7 @@
   with this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
-// $Id: sim_support.cpp 1665 2008-01-10 20:42:26Z stj2 $
+// $Id$
 #include <stdlib.h>
 #include <assert.h>
 #include <stdio.h>
@@ -54,6 +54,7 @@ extern "C" {
             struct sockaddr **from);
     void nsc_soconnect(void *so, int port, unsigned int addr);
     void nsc_soclose(void *so);
+    int nsc_getsockpeername(void *so, void *addrlen, size_t *addrspace, int *port, int peername);
     int nsc_sysctl(const char *sysctl_name, void *oldval, size_t *oldlenp,
         void *newval, size_t newlen);
 
@@ -85,6 +86,7 @@ extern "C" {
     void timer_expire(void *cc);
 
     /* support/support.c */
+    int nsc_convert_syserr_to_nscerr(int ret);
 };
 
 //! The mapping type of interface pointer -> BSDStack.
@@ -414,7 +416,8 @@ int BSDStack::sysctl(const char *sysctl_name, void *oldval, size_t *oldlenp,
     void *newval, size_t newlen)
 {
     set_stack_id(stack_id);
-    return nsc_sysctl(sysctl_name, oldval, oldlenp, newval, newlen);
+    int ret = nsc_sysctl(sysctl_name, oldval, oldlenp, newval, newlen);
+    return nsc_convert_syserr_to_nscerr(ret);
 }
 
 void BSDStack::buffer_size(int size)
@@ -550,8 +553,29 @@ int BSDStack::TCPSocket::accept(INetStreamSocket **sock)
     }
 
     unlock_stack();
+    return nsc_convert_syserr_to_nscerr(err);
+}
 
-    return err;
+int BSDStack::TCPSocket::getsockname(void *addrspc, size_t *len, int *port)
+{
+    int err;
+
+    lock_stack();
+    set_stack_id(parent->stack_id);
+    err = nsc_getsockpeername(so, addrspc, len, port, 0);
+    unlock_stack();
+    return nsc_convert_syserr_to_nscerr(err);
+}
+
+int BSDStack::TCPSocket::getpeername(void *addrspc, size_t *len, int *port)
+{
+    int err;
+
+    lock_stack();
+    set_stack_id(parent->stack_id);
+    err = nsc_getsockpeername(so, addrspc, len, port, 1);
+    unlock_stack();
+    return nsc_convert_syserr_to_nscerr(err);
 }
 
 int BSDStack::TCPSocket::send_data(const void *data, int datalen)
@@ -583,10 +607,10 @@ int BSDStack::TCPSocket::send_data(const void *data, int datalen)
     if(datalen > 0)
         return datalen;
     
-    return error > 0 ? -error : error;
+    return nsc_convert_syserr_to_nscerr(error);
 }
 
-int BSDStack::TCPSocket::read_data(const void *buf, int *buflen)
+int BSDStack::TCPSocket::read_data(void *buf, int *buflen)
 {
     int err;
 
@@ -597,7 +621,7 @@ int BSDStack::TCPSocket::read_data(const void *buf, int *buflen)
 
     unlock_stack();
 
-    return err;
+    return nsc_convert_syserr_to_nscerr(err);
 }
 
 /* Why does this even return a value? tcpsetsockopt doesn't. */
@@ -611,7 +635,7 @@ int BSDStack::TCPSocket::setsockopt(char *optname, void *val, size_t valsize)
     //tcpsetsockopt(so, optname, val, valsize);
     unlock_stack();
 
-    return 0;
+    return nsc_convert_syserr_to_nscerr(error);
 }
 
 void BSDStack::TCPSocket::disconnect()
