@@ -64,10 +64,12 @@ class BSDStack : public INetStack {
             virtual ~TCPSocket();
             virtual void connect(const char *, int);
             virtual void disconnect();
+            virtual int getpeername(void *addrspc, size_t *len, int *port);
+            virtual int getsockname(void *addrspc, size_t *len, int *port);
             virtual void listen(int);
             virtual int accept(INetStreamSocket **);
             virtual int send_data(const void *data, int datalen);
-            virtual int read_data(const void *buf, int *buflen);
+            virtual int read_data(void *buf, int *buflen);
             virtual int setsockopt(char *optname, void *val, size_t valsize);
             virtual void print_state(FILE *);
             virtual bool is_connected();
