@@ -17,7 +17,7 @@
   Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-/* $Id: support2.c 1609 2007-05-09 20:33:22Z stj2 $ */
+/* $Id$ */
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/socketvar.h>
@@ -170,7 +170,7 @@ int nsc_setsockopt(void *socket, char *optname, void *val, size_t valsize)
 
     if( strcmp(optname, "SO_SNDBUF") == 0 ) opt = SO_SNDBUF;
     else if( strcmp(optname, "SO_RCVBUF") == 0 ) opt = SO_RCVBUF;
-    else return 1;
+    else return -1;
     
     sopt.sopt_name = opt;
     sopt.sopt_dir = SOPT_SET;
