@@ -32,6 +32,7 @@
 
 #include "sim_interface.h"
 #include "nsc.h"
+#include "sysctl.h"
 #include "num_stacks.h"
 
 extern "C" {
@@ -43,6 +44,7 @@ extern "C" {
     void nsc_sysctl_a();
 
     void nsc_gettime(unsigned int *sec, unsigned int *usec);
+    void nsc_set_cpuset_netclass_id(int id);
 }
 // ------------------------------------------------------------------------
 // globals
@@ -116,28 +118,49 @@ private:
         virtual int accept(INetStreamSocket **sock)
         {
             void *s;
+            int ret;
             
             set_stack_id(parent->stack_id);
 
-            if(nsc_accept(so, &s) == 0) {
+            ret = nsc_accept(so, &s);
+            if (ret == 0) {
                 *sock = new TCPSocket(parent, s);
                 return 0;
             }
             set_stack_id(-1);
-
-            return EAGAIN; 
+	    assert(ret < 0);
+            return nsc_convert_syserr_to_nscerr(ret);
         }
-        
+	virtual int getpeername(void *addrspace, size_t *addrspace_len, int *port)
+	{
+		int ret;
+		set_stack_id(parent->stack_id);
+		ret = nsc_getsockpeername(so, addrspace, addrspace_len, port, 1);
+		set_stack_id(-1);
+		return ret;
+	}
+
+	virtual int getsockname(void *addrspace, size_t *addrspace_len, int *port)
+	{
+		int ret;
+		set_stack_id(parent->stack_id);
+		ret = nsc_getsockpeername(so, addrspace, addrspace_len, port, 0);
+		set_stack_id(-1);
+		return ret;
+	}
+
         virtual int send_data(const void *data, int datalen)
         {
             set_stack_id(parent->stack_id);
-            return nsc_sosend(so, (void *)data, datalen);
+            int ret = nsc_sosend(so, (void *)data, datalen);
+            return nsc_convert_syserr_to_nscerr(ret);
         }
         
-        virtual int read_data(const void *buf, int *buflen)
+        virtual int read_data(void *buf, int *buflen)
         {
             set_stack_id(parent->stack_id);
-            return nsc_soreceive(so, (void *)buf, buflen);
+            int ret = nsc_soreceive(so, buf, buflen);
+            return nsc_convert_syserr_to_nscerr(ret);
         }
         
         virtual int setsockopt(char *optname, void *val, size_t valsize) 
@@ -155,7 +178,8 @@ private:
             }
             
             set_stack_id(parent->stack_id);
-            return nsc_setsockopt(so, optname, val, valsize);
+            int ret = nsc_setsockopt(so, optname, val, valsize);
+	    return nsc_convert_syserr_to_nscerr(ret);
         }
         
         virtual void print_state(FILE *) 
@@ -210,16 +234,17 @@ private:
         virtual int accept(INetStreamSocket **sock)
         {
             void *s;
-            
+            int ret;
             set_stack_id(parent->stack_id);
 
-            if(nsc_accept_sctp(so, &s) == 0) {
+            ret = nsc_accept_sctp(so, &s);
+            if (ret == 0) {
                 *sock = new SCTPSocket(parent, s);
                 return 0;
             }
 
-            return EAGAIN; 
-
+	    assert(ret < 0);
+	    return nsc_convert_syserr_to_nscerr(ret);
         }
         
         int dummy;
@@ -288,6 +313,9 @@ public:
 
     virtual void if_send_finish(int if_id)
     {
+      set_stack_id(stack_id);
+      nsc_if_xmit_finished(interfaces[if_id]);
+      // set_stack_id(-1); // we can't do this here, same as if_send_packet
     }
 
     virtual void if_attach(const char *addr, const char *mask, int mtu)
@@ -304,11 +332,12 @@ public:
 
     virtual void add_default_gateway(const char *addr)
     {
-        unsigned int ip_addr;
-        inet_aton(addr, (struct in_addr *)&ip_addr);
+        unsigned int iaddr;
+
+        inet_aton(addr, (struct in_addr *)&iaddr); 
 
         set_stack_id(stack_id);
-        nsc_add_default_gw(ip_addr);
+        nsc_add_default_gw(iaddr);
         set_stack_id(-1);
     }
 
@@ -317,21 +346,11 @@ public:
         return stack_id;
     }
 
-    virtual const char *get_name()
-    {
-        return "linux26";
-    }
+    virtual const char *get_name() { return "linux2.6.18"; }
 
     virtual int get_hz()
     {
-        return 1000;
-    }
-
-    virtual int get_ticks()
-    {
-        // we could return global_tcp_ticks or something if we cared. But
-        // I don't see why I even have this function...
-        return 0;
+        return nsc_get_hz();
     }
 
     virtual void timer_interrupt()
@@ -341,14 +360,6 @@ public:
         set_stack_id(-1);
     }
 
-    virtual void nsi_interrupt()
-    {
-    }
-
-    virtual void wakeup(void *)
-    {
-    }
-    
     virtual void increment_ticks()
     {
         set_stack_id(stack_id);
@@ -356,28 +367,37 @@ public:
         set_stack_id(-1);
     }
     
-    virtual struct INetDatagramSocket *new_udp_socket()
-    {
-        assert(0 && "UDP is not yet supported!");
-        return NULL; 
-    }
-    
     virtual struct INetStreamSocket *new_tcp_socket()
     {
         set_stack_id(stack_id);
         return new TCPSocket(this);
-
-        // The following allows us to use SCTP in the place of TCP!
-        //return new SCTPSocket(this);
     }
 
     virtual int sysctl(const char *sysctl_name, void *oldval, size_t *oldlenp,
         void *newval, size_t newlen)
     {
         set_stack_id(stack_id);
-        return nsc_do_sysctl(sysctl_name, oldval, oldlenp, newval, newlen);
+        int ret = nsc_do_sysctl(sysctl_name, oldval, oldlenp, newval, newlen);
+        return nsc_convert_syserr_to_nscerr(ret);
     }
 
+    virtual int sysctl_set(const char *name, const char *value)
+    {
+	    set_stack_id(stack_id);
+	    return nsc_sysctl_util_set(name, value);
+    }
+
+    virtual int sysctl_get(const char *name, char *value, size_t len)
+    {
+	    set_stack_id(stack_id);
+	    return nsc_sysctl_util_get(name, value, len);
+    }
+
+    virtual int sysctl_getnum(size_t idx, char *name, size_t len)
+    {
+	    set_stack_id(stack_id);
+	    return nsc_sysctl_util_getnum(idx, name, len);
+    }
 
     virtual void buffer_size(int size)
     {
@@ -390,13 +410,13 @@ public:
 
         newval = size;
         oldlen = sizeof(oldval);
-        err = sysctl("net/core/wmem_max", &oldval, &oldlen, &newval, newlen);
+        err = sysctl("net.core.wmem_max", &oldval, &oldlen, &newval, newlen);
         //fprintf(stderr, "sysctl returned %d (%d, %d)\n", err, oldval, newval);
         assert(err == 0);
 
         newval = size;
         oldlen = sizeof(oldval);
-        err = sysctl("net/core/rmem_max", &oldval, &oldlen, &newval, newlen);
+        err = sysctl("net.core.rmem_max", &oldval, &oldlen, &newval, newlen);
         //fprintf(stderr, "sysctl returned %d (%d, %d)\n", err, oldval, newval);
         assert(err == 0);
 
@@ -406,7 +426,7 @@ public:
         newval3[1] = size;
         newval3[2] = size;
         oldlen = sizeof(oldval3);
-        err = sysctl("net/ipv4/tcp_rmem", oldval3, &oldlen, newval3, newlen);
+        err = sysctl("net.ipv4.tcp_rmem", oldval3, &oldlen, newval3, newlen);
         /*fprintf(stderr, "sysctl returned %d ([%d,%d,%d], [%d,%d,%d])\n", 
                 err, 
                 oldval3[0], oldval3[1], oldval3[2], 
@@ -418,7 +438,7 @@ public:
         newval3[1] = size;
         newval3[2] = size;
         oldlen = sizeof(oldval3);
-        err = sysctl("net/ipv4/tcp_wmem", oldval3, &oldlen, newval3, newlen);
+        err = sysctl("net.ipv4.tcp_wmem", oldval3, &oldlen, newval3, newlen);
         /*fprintf(stderr, "sysctl returned %d ([%d,%d,%d], [%d,%d,%d])\n", 
                 err, 
                 oldval3[0], oldval3[1], oldval3[2], 
@@ -444,6 +464,36 @@ public:
     {
         nsc_g_l26_diagnostic = level;
     }
+
+    virtual int cmd(const char *c)
+    {
+      const char set_cpuset_netclass_id[] = "set_cpuset_netclass_id ";
+      const char tc[] = "tc ";
+
+      if (strncmp(c, tc, strlen(tc)) == 0)
+      {
+        set_stack_id(stack_id);
+        nsc_tc(c);
+        set_stack_id(-1);
+
+        return 0;
+      }
+      else if (strncmp(c, set_cpuset_netclass_id, 
+            strlen(set_cpuset_netclass_id)) == 0)
+      {
+        const char *num_str = c + strlen(set_cpuset_netclass_id);
+        int num = strtoul(num_str, NULL, 0);
+
+        set_stack_id(stack_id);
+        nsc_set_cpuset_netclass_id(num);
+        set_stack_id(-1);
+
+        return 0;
+      }
+
+
+      return 1;
+    }
 };
 
 CREATE_STACK_FUNC(send_callback, int_callback, rand_func)
@@ -538,15 +588,23 @@ extern "C" void panic(const char * a, ...)
     while(1) ;
 }
 
+/* 
+ * added support for electric fence memory debbuger
+ */
+extern "C" void *ef_malloc(size_t size);
+extern "C" void ef_free(void *addr);
+
 extern "C" void *nsc_malloc(int size)
 {
-    void *s = malloc(size);
-    return s;
+  //    void *s = ef_malloc(size);
+  void *s = malloc(size);
+  return s;
 }
 
 extern "C" void nsc_free(void *o)
 {
-    free(o);
+  //    ef_free(o);
+  free(o);
 }
 
 extern "C" void *nsc_memcpy(void *to, const void *from, unsigned len)
