@@ -74,6 +74,7 @@ extern void nsc_assert(int, const char *);
 #include <net/xfrm.h>
 #include <linux/crypto.h>
 #include <linux/seq_file.h>
+#include <linux/ticable.h>
 
 #undef __FUNCTION__
 
@@ -133,7 +134,9 @@ int seq_printf(struct seq_file *m, const char *f, ...)
     UNIMPLEMENTED();
 }*/
 
-//inline void prefetch(const void *x) {;} /* Fdb */
+/* defined in include/asm/processor.h 
+inline void prefetch(const void *x) {;}
+*/
 
 /*void __lockfunc _write_lock(rwlock_t *lock)
 {
@@ -282,12 +285,6 @@ unsigned long simple_strtoul(const char *cp,char **endp,unsigned int base)
     return 0;
 }
 
-int request_module(const char *fmt, ...)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
 struct crypto_tfm *crypto_alloc_tfm(const char *name, u32 flags)
 {
     UNIMPLEMENTED();
@@ -307,7 +304,7 @@ xfrm_state_lookup(xfrm_address_t *daddr, u32 spi, u8 proto,
     return 0;
 }
 
-int fastcall wake_up_process(task_t * p)
+int fastcall wake_up_process(struct task_struct *p)
 {
     UNIMPLEMENTED();
     return 0;
@@ -327,11 +324,13 @@ int seq_putc(struct seq_file *m, char c)
     return 0;
 }
 
+/* in implemented.c 
 int notifier_chain_unregister(struct notifier_block **nl, struct notifier_block *n)
 {
     UNIMPLEMENTED();
     return 0;
 }
+*/
 
 spinlock_t kernel_flag;
 
@@ -339,11 +338,13 @@ cpumask_t cpu_online_map = CPU_MASK_ALL;
 
 struct pglist_data contig_page_data/* = { .bdata = &contig_bootmem_data }*/;
 
+/*
 void fastcall call_rcu(struct rcu_head *head,
 				void (*func)(struct rcu_head *rcu))
 {
     UNIMPLEMENTED();
 }
+*/
 
 struct page * fastcall
 __alloc_pages(unsigned int gfp_mask, unsigned int order,
@@ -359,11 +360,6 @@ int xfrm4_rcv(struct sk_buff *skb)
     return 0;
 }
 
-void sysfs_remove_group(struct kobject * k, const struct attribute_group * g)
-{
-    UNIMPLEMENTED();
-}
-
 asmlinkage long sys_close(unsigned int fd)
 {
     UNIMPLEMENTED();
@@ -377,12 +373,6 @@ void fastcall put_unused_fd(unsigned int fd)
 
 struct proc_dir_entry *proc_sys_root;
 
-int find_next_bit(const unsigned long *addr, int size, int offset)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
 void crypto_hmac(struct crypto_tfm *tfm, u8 *key, unsigned int *keylen,
                  struct scatterlist *sg, unsigned int nsg, u8 *out)
 {
@@ -436,6 +426,7 @@ int skb_cow_data(struct sk_buff *skb, int tailbits, struct sk_buff **trailer)
     return 0;
 }
 
+#ifdef LOCAL_RANDOM
 __u32 secure_tcpv6_sequence_number(__u32 *saddr, __u32 *daddr,
 				   __u16 sport, __u16 dport)
 {
@@ -448,12 +439,7 @@ __u32 secure_ip_id(__u32 daddr)
     UNIMPLEMENTED();
     return 0;
 }
-
-int fastcall schedule_work(struct work_struct *work)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
+#endif
 
 struct rw_semaphore fastcall *rwsem_wake(struct rw_semaphore *sem)
 {
@@ -478,16 +464,12 @@ void *pskb_put(struct sk_buff *skb, struct sk_buff *tail, int len)
     return 0;
 }
 
-long pps_valid = PPS_VALID;        /* pps signal watchdog counter */
-
 int kthread_should_stop(void)
 {
     UNIMPLEMENTED();
     return 0;
 }
 
-u64 jiffies_64;
-
 int ipv6_skip_exthdr(const struct sk_buff *skb, int start, u8 *nexthdrp)
 {
     UNIMPLEMENTED();
@@ -510,18 +492,18 @@ struct file fastcall *fget(unsigned int fd)
     return 0;
 }
 
+struct file fastcall *fget_light(unsigned int fd, int *fput_needed)
+{
+    UNIMPLEMENTED();
+    return 0;
+}
+
+
 void fastcall fd_install(unsigned int fd, struct file * file)
 {
     UNIMPLEMENTED();
 }
 
-/* Fred
-long div_ll_X_l_rem(long long divs, long div, long *rem)
-{
-    UNIMPLEMENTED();
-    return 0;
-}*/ 
-
 int default_wake_function(wait_queue_t *curr, unsigned mode, int sync, void *key)
 {
     UNIMPLEMENTED();
@@ -545,11 +527,6 @@ void crypto_hmac_final(struct crypto_tfm *tfm, u8 *key,
     UNIMPLEMENTED();
 }
 
-void clock_was_set(void)
-{
-    UNIMPLEMENTED();
-}
-
 void __kill_fasync(struct fasync_struct *fa, int sig, int band)
 {
     UNIMPLEMENTED();
@@ -593,7 +570,7 @@ int sysctl_lower_zone_protection = 0;
     return 0;
 }*/
 
-int simple_statfs(struct super_block *sb, struct kstatfs *buf)
+int simple_statfs(struct dentry *de, struct kstatfs *buf)
 {
     UNIMPLEMENTED();
     return 0;
@@ -612,7 +589,7 @@ size_t     shm_ctlall = SHMALL;
 
 int sg_big_buff/* = SG_DEF_RESERVED_SIZE*/;
 
-void set_user_nice(task_t *p, long nice)
+void set_user_nice(struct task_struct *p, long nice)
 {
     UNIMPLEMENTED();
 }
@@ -639,12 +616,6 @@ __u32 secure_tcp_syn_cookie(__u32 saddr, __u32 daddr, __u16 sport,
     return 0;
 }
 
-int fastcall schedule_delayed_work(struct work_struct *work, unsigned long delay)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
 struct rw_semaphore fastcall __sched *
 rwsem_down_write_failed(struct rw_semaphore *sem)
 {
@@ -659,7 +630,9 @@ rwsem_down_read_failed(struct rw_semaphore *sem)
     return 0;
 }
 
+#ifdef LOCAL_RANDOM
 ctl_table random_table[1] /* = { ... */;
+#endif
 
 void put_filp(struct file *file)
 {
@@ -671,6 +644,8 @@ ctl_table pty_table[1] /* = { ... */;
 int proc_unknown_nmi_panic(ctl_table *table, int write, struct file *file,
 			void __user *buffer, size_t *length, loff_t *ppos)
 {
+    printk("proc_unknown_nmi_panic: %p %d %p %p %d %d\n",
+            table, write, file, buffer, *length, (int) *ppos);
     UNIMPLEMENTED();
     return 0;
 }
@@ -681,7 +656,7 @@ int printk_ratelimit_burst = 10;
 
 long pps_stabil = MAXFREQ; /* frequency dispersion (scaled ppm) */
 
-long pps_jitter = MAXTIME; /* time dispersion (jitter) (us) */
+long pps_jitter = TIMAXTIME; /* time dispersion (jitter) (us) */
 
 long pps_freq;                     /* frequency offset (scaled ppm) */
 
@@ -714,8 +689,11 @@ int msg_ctlmax = MSGMAX;
 char modprobe_path[KMOD_PATH_LEN] = "/sbin/modprobe";
 
 int min_free_kbytes_sysctl_handler(ctl_table *table, int write, 
-		struct file *file, void __user *buffer, size_t *length, loff_t *ppos)
+		struct file *file, void __user *buffer, size_t *length, 
+                loff_t *ppos)
 {
+    printk("min_free_kbytes_sysctl_handler: %p %d %p %p %d %d\n",
+            table, write, file, buffer, *length, (int) *ppos);
     UNIMPLEMENTED();
     return 0;
 }
@@ -727,6 +705,8 @@ int max_threads;
 int lower_zone_protection_sysctl_handler(ctl_table *table, int write,
 		 struct file *file, void __user *buffer, size_t *length, loff_t *ppos)
 {
+    printk("lower_zone_protection_sysctl_handler: %p %d %p %p %d %d\n",
+            table, write, file, buffer, *length, (int) *ppos);
     UNIMPLEMENTED();
     return 0;
 }
@@ -805,9 +785,10 @@ void getnstimeofday(struct timespec *tv)
     UNIMPLEMENTED();
 }
 
-struct super_block *
+int
 get_sb_pseudo(struct file_system_type *fs_type, char *name,
-	struct super_operations *ops, unsigned long magic)
+	struct super_operations *ops, unsigned long magic,
+	struct vfsmount *mnt)
 {
     UNIMPLEMENTED();
     return 0;
@@ -838,6 +819,9 @@ int f_setown(struct file *filp, unsigned long arg, int force)
 int dirty_writeback_centisecs_handler(ctl_table *table, int write,
 		struct file *file, void __user *buffer, size_t *length, loff_t *ppos)
 {
+    //UNIMPLEMENTED();
+    printk("dirty_writeback_centisecs_handler: %p %d %p %p %d %d\n",
+            table, write, file, buffer, *length, (int) *ppos);
     UNIMPLEMENTED();
     return 0;
 }
@@ -870,7 +854,7 @@ struct dentry *d_alloc(struct dentry * parent, const struct qstr *name)
 
 int core_uses_pid;
 
-char core_pattern[65] = "core";
+char core_pattern[] = "core";
 
 int __bitmap_weight(const unsigned long *bitmap, int bits)
 {
@@ -880,11 +864,6 @@ int __bitmap_weight(const unsigned long *bitmap, int bits)
 
 int C_A_D = 1;
 
-void __sched __cond_resched(void)
-{
-    UNIMPLEMENTED();
-}
-
 unsigned long
 __copy_from_user_ll(void *to, const void __user *from, unsigned long n)
 {
@@ -905,8 +884,8 @@ fastcall void __free_pages(struct page *page, unsigned int order)
 
 int __printk_ratelimit(int ratelimit_jiffies, int ratelimit_burst)
 {
-    UNIMPLEMENTED();
-    return 0;
+  //    UNIMPLEMENTED();
+    return 1;
 }
 
 void fastcall add_wait_queue(wait_queue_head_t *q, wait_queue_t * wait)
@@ -919,9 +898,9 @@ void fastcall add_wait_queue_exclusive(wait_queue_head_t *q, wait_queue_t * wait
     UNIMPLEMENTED();
 }
 
-unsigned aio_max_nr = 0x10000;      /* system wide maximum number of aio requests */
+unsigned long aio_max_nr = 0x10000;      /* system wide maximum number of aio requests */
 
-atomic_t aio_nr = ATOMIC_INIT(0);   /* current system wide number of aio requests */
+unsigned long aio_nr = 0;   /* current system wide number of aio requests */
 
 int block_dump;
 
@@ -960,22 +939,10 @@ void fastcall complete(struct completion *x)
     UNIMPLEMENTED();
 }
 
-int cap_netlink_send (struct sock *sk, struct sk_buff *skb)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
-int cap_netlink_recv (struct sk_buff *skb)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
 
 int console_printk[4]; /* normally initialised here. kernel/printk.c */
 
-DEFINE_PER_CPU(struct rcu_data, rcu_bh_data) = { 0L };
+// DEFINE_PER_CPU(struct rcu_data, rcu_bh_data) = { 0L };
 
 unsigned char _ctype[256]; /* should be initialised */
 
@@ -1045,161 +1012,548 @@ void __init kmem_cache_init(void)
 
 }
 
-
-irq_cpustat_t irq_stat[NR_CPUS] ____cacheline_aligned;
-EXPORT_SYMBOL(irq_stat);
-
-//#include <linux/netfilter.h>
-//struct list_head nf_hooks[NPROTO][NF_MAX_HOOKS];
-//EXPORT_SYMBOL(nf_hooks);
-
+/* added from 2.6.14.2 as needed */
 int lowmem_reserve_ratio_sysctl_handler(ctl_table *table, int write,
-                struct file *file, void __user *buffer, size_t *length, loff_t *ppos)
+		                struct file *file, void __user *buffer, size_t *length, loff_t *ppos)
 {
-    UNIMPLEMENTED();
-    return 0;
+	    UNIMPLEMENTED();
+	        return 0;
 }
 
 int sysctl_lowmem_reserve_ratio[MAX_NR_ZONES-1]; // = { 256, 256, 32 };
 
-
-int param_set_int(const char *val, struct kernel_param *kp) 
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
-int param_get_int(char *buffer, struct kernel_param *kp)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
-
-DEFINE_PER_CPU(irq_cpustat_t, irq_stat);
-
-int suid_dumpable = 0;
-
 int bootloader_type;
 
-int __sched cond_resched_softirq(void)
+/* int __sched cond_resched_softirq(void)
 {
-    UNIMPLEMENTED();
-    return 0;
-}
-
-void set_process_cpu_timer(struct task_struct *tsk, unsigned int clock_idx,
-                           cputime_t *newval, cputime_t *oldval)
-{
-    UNIMPLEMENTED();
-}
-
-task_t fastcall *next_thread(const task_t *p)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
+	    UNIMPLEMENTED();
+	        return 0;
+} */
 
 int cond_resched_lock(spinlock_t *lock)
 {
-    UNIMPLEMENTED();
-    return 0;
+	    UNIMPLEMENTED();
+	        return 0;
 }
 
 long
 strncpy_from_user(char *dst, const char __user *src, long count)
 {
-    UNIMPLEMENTED();
-    return 0;
-}
-    
-void sha_transform(__u32 *digest, const char *in, __u32 *W)
-{
-    UNIMPLEMENTED();
-}
-
-u32 secure_tcpv6_port_ephemeral(const __u32 *saddr, const __u32 *daddr, __u16 dport)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
-void run_posix_cpu_timers(struct task_struct *tsk)
-{
-    UNIMPLEMENTED();
+	    UNIMPLEMENTED();
+	        return 0;
 }
 
 /* Definition for rcupdate control block. */
 struct rcu_ctrlblk rcu_ctrlblk = {
-        .cur = -300,
-        .completed = -300,
+	        .cur = -300,
+		.completed = -300,
 //        .lock = SPIN_LOCK_UNLOCKED,
 //        .cpumask = CPU_MASK_NONE,
 };
 struct rcu_ctrlblk rcu_bh_ctrlblk = {
-        .cur = -300,
-        .completed = -300,
+       .cur = -300,
+       .completed = -300,
 //        .lock = SPIN_LOCK_UNLOCKED,
 //        .cpumask = CPU_MASK_NONE,
 };
 
-void rcu_check_callbacks(int cpu, int user)
-{
-    UNIMPLEMENTED();
-}
-
-DEFINE_PER_CPU(struct rcu_data, rcu_data) = { 0L };
-
-int audit_socketcall(int nargs, unsigned long *args)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
-int audit_sockaddr(int len, void *a)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
-
-uid_t audit_get_loginuid(struct audit_context *ctx)
-{
-    UNIMPLEMENTED();
-    return 0;
-}
+// DEFINE_PER_CPU(struct rcu_data, rcu_data) = { 0L };
 
 void account_user_time(struct task_struct *p, cputime_t cputime)
 {
-    UNIMPLEMENTED();
+	    UNIMPLEMENTED();
 }
 
 void account_system_time(struct task_struct *p, int hardirq_offset,
-        cputime_t cputime)
+		        cputime_t cputime)
 {
-    UNIMPLEMENTED();
+	    UNIMPLEMENTED();
 }
 
-int __sched cond_resched(void)
+/* new */
+int (*inferior_user)(int, uid_t) = NULL;
+
+void get_netdump_dest(char *buff, int buffsize, u32 *dest, u32 *src)
 {
-    UNIMPLEMENTED();
-    return 0;
+  UNIMPLEMENTED();
 }
 
+void set_netdump_dest(struct net_device *d, u32 dest_ip, u32 src_ip)
+{
+  UNIMPLEMENTED();
+}
+  
+int network_dumper_one_shot;
+
+int network_dumper_xmit_delay;
+
+unsigned long shm_page_cache_evictions;
+
+int sysctl_oom_kill_adjust_root;
+
+
 #include <linux/init.h>
 void __init do_initcalls(void)
 {
     // XXX: 2.6: fix this function
     initcall_t *call;
-    extern initcall_t __initcall_start, __initcall_end;
+    extern initcall_t __initcall_start[], __initcall_end[];
 
-    call = &__initcall_start;
+    call = __initcall_start;
     do {
         (*call)();
         call++;
-    } while (call < &__initcall_end);
+    } while (call < __initcall_end);
 
     /* Make sure there is no pending stuff from the initcall sequence
 */
     //flush_scheduled_tasks();
 }
 
+#include <linux/device.h>
+void class_device_initialize(struct class_device *cd)
+{
+	UNIMPLEMENTED();
+}
+
+int class_device_add(struct class_device *cd)
+{
+	UNIMPLEMENTED();
+	return 0;
+}
+
+struct sock *xfrm_nl=NULL;
+
+void xfrm_replay_notify (struct xfrm_state *x, int event)
+{
+	UNIMPLEMENTED();
+}
+
+void __mark_inode_dirty (struct inode *inode, int val)
+{
+	UNIMPLEMENTED();
+}
+
+unsigned long acpi_video_flags;
+
+struct timespec current_fs_time(struct super_block *sb)
+{
+	static struct timespec ts;
+	UNIMPLEMENTED ();
+	return ts;
+}
+
+int dirty_expire_interval;
+int dirty_writeback_interval;
+
+int drop_caches_sysctl_handler(struct ctl_table *a1, int a2, struct file *a3,
+		 	       void __user *a4, size_t *a5, loff_t *a6)
+{
+	UNIMPLEMENTED();
+	return 0;
+}
+
+void run_posix_cpu_timers(struct task_struct *task)
+{
+	UNIMPLEMENTED();
+}
+
+ssize_t generic_splice_sendpage(struct pipe_inode_info *pipe,
+		struct file *out, loff_t *loff, size_t len, 
+		unsigned int flags)
+{
+	ssize_t size=1;
+	
+	UNIMPLEMENTED();
+	return size;
+}
+
+int randomize_va_space=0;
+
+int suid_dumpable;
+
+unsigned long hrtimer_forward(struct hrtimer *timer, ktime_t now,
+			ktime_t interval)
+{
+	UNIMPLEMENTED();
+	return 0;
+}
+
+ktime_t hrtimer_get_remaining(const struct hrtimer *timer)
+{
+	ktime_t t;
+	UNIMPLEMENTED();
+	return t;
+}
+
+void hrtimer_run_queues(void)
+{
+    
+}
+
+int hrtimer_start(struct hrtimer *timer, ktime_t tim, 
+		const enum hrtimer_mode mode)
+{
+	UNIMPLEMENTED();
+	return 1;
+}
+
+int hrtimer_try_to_cancel(struct hrtimer *timer)
+{
+	UNIMPLEMENTED();
+	return 1;
+}
+
+int idle_cpu(int cpu)
+{
+	UNIMPLEMENTED();
+	return 0;
+}
+
+cpumask_t nohz_cpu_mask;
+
+int percpu_pagelist_fraction;
+
+int percpu_pagelist_fraction_sysctl_handler(struct ctl_table *tbl,
+			int val, struct file *fp, void __user *u,
+			size_t *size, loff_t *off)
+{
+	UNIMPLEMENTED();
+	return 1;
+}
+
+int proc_nr_files(ctl_table *table, int write, struct file *filp,
+		void __user *buffer, size_t *lenp, loff_t *ppos)
+{
+	UNIMPLEMENTED();
+	return 1;
+}
+
+struct rw_semaphore fastcall *rwsem_downgrade_wake(struct rw_semaphore *sem)
+{
+	UNIMPLEMENTED();
+	return sem;
+}
+
+void set_process_cpu_timer(struct task_struct *tsk, unsigned int clock_idx,
+		cputime_t *newval, cputime_t *oldval)
+{
+	UNIMPLEMENTED();
+}
+
+/* In implemented
+void softlockup_tick(void)
+{
+	UNIMPLEMENTED();
+}
+*/
+
+int sysctl_drop_caches=0;
+int sysctl_panic_on_oom=0;
+
+int sysdev_class_register(struct sysdev_class *class)
+{
+	/* UNIMPLEMENTED(); */
+	return 1;
+}
+
+int sysdev_register(struct sys_device *dev)
+{
+	/* UNIMPLEMENTED(); */
+	return 1;
+}
+
+void touch_atime(struct vfsmount *mnt, struct dentry *dentry)
+{
+	UNIMPLEMENTED();
+}
+
+char uevent_helper[100] = "SIM ERROR";
+
+struct ctl_table ether_table[100];
+
+// The following are new as of Oct-12-07, will need some work yet
+
+struct dentry * lookup_one_len(const char * name, struct dentry * base, int len)
+{
+  UNIMPLEMENTED();
+  return NULL; // TODO
+}
+
+void d_delete(struct dentry * dentry)
+{
+  UNIMPLEMENTED();
+}
+
+int simple_rmdir(struct inode *dir, struct dentry *dentry)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+void dput(struct dentry *dentry)
+{
+  UNIMPLEMENTED();
+}
+
+struct dentry * dget_locked(struct dentry *dentry)
+{
+  UNIMPLEMENTED();
+  return NULL;
+}
+
+int simple_unlink(struct inode *dir, struct dentry *dentry)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+/*
+ * Lookup the data. This is trivial - if the dentry didn't already
+ * exist, we know it is negative.  Set d_op to delete negative dentries.
+ */
+struct dentry *simple_lookup(struct inode *dir, struct dentry *dentry, struct nameidata *nd)
+{
+  UNIMPLEMENTED();
+  return NULL;
+}
+
+struct inode_operations simple_dir_inode_operations = {
+        .lookup         = simple_lookup,
+};
+
+const struct file_operations simple_dir_operations = {
+/*        .open           = dcache_dir_open,
+        .release        = dcache_dir_close,
+        .llseek         = dcache_dir_lseek,
+        .read           = generic_read_dir,
+        .readdir        = dcache_readdir,
+        .fsync          = simple_sync_file,*/
+};
+
+struct dentry * d_alloc_root(struct inode * root_inode)
+{
+  UNIMPLEMENTED();
+  return NULL;
+}
+
+int get_sb_single(struct file_system_type *fs_type,
+        int flags, void *data,
+        int (*fill_super)(struct super_block *, void *, int),
+        struct vfsmount *mnt)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int call_usermodehelper_keys(char *path, char **argv, char **envp,
+                             struct key *session_keyring, int wait)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+struct task_struct *find_task_by_pid_type(int type, int nr)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+ssize_t simple_read_from_buffer(void __user *to, size_t count, loff_t *ppos,
+                                const void *from, size_t available)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+void __bitmap_and(unsigned long *dst, const unsigned long *bitmap1,
+                 const unsigned long *bitmap2, int bits)
+{
+  UNIMPLEMENTED();
+}
+
+void __bitmap_andnot(unsigned long *dst, const unsigned long *bitmap1,
+                 const unsigned long *bitmap2, int bits)
+{
+  UNIMPLEMENTED();
+}
+
+int __bitmap_empty(const unsigned long *bitmap, int bits)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int __bitmap_equal(const unsigned long *bitmap1,
+                   const unsigned long *bitmap2, int bits)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int __bitmap_intersects(const unsigned long *bitmap1,
+                        const unsigned long *bitmap2, int bits)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+void __bitmap_or(unsigned long *dst, const unsigned long *bitmap1,
+                 const unsigned long *bitmap2, int bits)
+{
+  UNIMPLEMENTED();
+}
+
+int __bitmap_subset(const unsigned long *bitmap1,
+                 const unsigned long *bitmap2, int bits)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+nodemask_t node_online_map;
+
+int partition_sched_domains(cpumask_t *partition1, cpumask_t *partition2)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int bitmap_parselist(const char *buf, unsigned long *maskp,
+                     int nmaskbits)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int bitmap_scnlistprintf(char *buf, unsigned int len,
+                         const unsigned long *src, int nbits)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int bitmap_scnprintf(char *buf, unsigned int len,
+                     const unsigned long *src, int nbits)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+struct task_struct init_task;
+
+struct mm_struct *get_task_mm(struct task_struct *task)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+void mmput(struct mm_struct *mm)
+{
+  UNIMPLEMENTED();
+}
+
+void __put_task_struct(struct task_struct *tsk)
+{
+  UNIMPLEMENTED();
+}
+
+void generic_delete_inode(struct inode *inode)
+{
+  UNIMPLEMENTED();
+}
+
+loff_t generic_file_llseek(struct file *file, loff_t offset, int origin)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int generic_file_open(struct inode * inode, struct file * filp)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int simple_rename(struct inode *old_dir, struct dentry *old_dentry,
+                  struct inode *new_dir, struct dentry *new_dentry)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+struct task_struct *fastcall get_pid_task(struct pid *pid, enum pid_type type)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+void kill_litter_super(struct super_block *sb)
+{
+  UNIMPLEMENTED();
+}
+
+void sort(void *base, size_t num, size_t size,
+          int (*cmp)(const void *, const void *),
+          void (*swap)(void *, void *, int size))
+{
+  UNIMPLEMENTED();
+}
+
+void fastcall __down_read(struct rw_semaphore *sem)
+{
+  UNIMPLEMENTED();
+}
+
+int fastcall __down_read_trylock(struct rw_semaphore *sem)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+void fastcall __down_write(struct rw_semaphore *sem)
+{
+  UNIMPLEMENTED();
+}
+
+int fastcall __down_write_trylock(struct rw_semaphore *sem)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+void fastcall __downgrade_write(struct rw_semaphore *sem)
+{
+  UNIMPLEMENTED();
+}
+
+void fastcall __up_read(struct rw_semaphore *sem)
+{
+  UNIMPLEMENTED();
+}
+
+void fastcall __up_write(struct rw_semaphore *sem)
+{
+  UNIMPLEMENTED();
+}
+
+
+int param_set_int(const char *val, struct kernel_param *kp)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+int param_get_int(char *buffer, struct kernel_param *kp)
+{
+  UNIMPLEMENTED();
+  return 0;
+}
+
+DEFINE_PER_CPU(irq_cpustat_t, irq_stat);
+
+irq_cpustat_t irq_stat[NR_CPUS];
+
+#ifdef __x86_64
+seqlock_t xtime_lock;
+#endif
