@@ -37,6 +37,7 @@
 #include <linux/proc_fs.h>
 
 #include "../nsc/nsc.h"
+#include "sim_errno.h"
 
 extern void sock_init(void);
 extern void do_initcalls(void);
@@ -100,7 +101,7 @@ extern volatile unsigned long jiffies;
 
 void nsc_timer_tick(void)
 {
-  //    extern volatile unsigned long jiffies;
+//    extern volatile unsigned long jiffies;
     struct pt_regs regs;
     memset(&regs, 0, sizeof(regs));
     //nsc_debugf("nsc_timer_tick\n");
@@ -166,7 +167,7 @@ static void nsc_if_set_addr(unsigned int ip_addr, unsigned int ip_mask)
 
 }
 
-void nsc_if_receive_packet(void *dev, void *data, unsigned int datalen)
+void nsc_if_receive_packet(void *dev, const void *data, unsigned int datalen)
 {
     struct sk_buff *skb;
     // To be consistent with the eepro100 driver.
@@ -197,7 +198,7 @@ void nsc_if_receive_packet(void *dev, void *data, unsigned int datalen)
 
     skb->dev = (struct net_device *)dev;
     skb->protocol = 0x08; // IP
-    skb->ip_summed = CHECKSUM_UNNECESSARY; // Stops the TCP checksum
+    //skb->ip_summed = CHECKSUM_UNNECESSARY; // Stops the TCP checksum
     
     // _ni means "non-interrupt"
     netif_rx_ni(skb);
@@ -261,7 +262,7 @@ void *nsc_socreate_tcp()
     nsc_assert(
             sock_create(AF_INET, SOCK_STREAM, IPPROTO_TCP, &sock)
             == 0, "sock_create failed in nsc_socreate_tcp");
-    //sock->passcred = 0; /* EDIT: Fred for NSC: doesn't exist anymore in 2.6.14.2? */
+    sock->passcred = 0;
 
     return sock;
 }
@@ -641,108 +642,103 @@ int nsc_do_sysctl(const char *sysctl_name, void *oldval, size_t *oldlenp,
             name[nlen++] = NET_IPV4;
         } else if(strncmp(s, "core/", 5) == 0) {
             name[nlen++] = NET_CORE;
-/*      } else if(strncmp(s, "tcp_westwood", 12) == 0) {
+        } else if(strncmp(s, "tcp_westwood", 12) == 0) {
             name[nlen++] = NET_TCP_WESTWOOD;
             // net/ipv4/tcp_westwood
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
-*/ /* EDIT: Fred for NSC: No longer present in >= 2.6.13 */
         } else if(strncmp(s, "wmem_max", 8) == 0) {
             name[nlen++] = NET_CORE_WMEM_MAX;
             // net/core/wmem_max
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "rmem_max", 8) == 0) {
             name[nlen++] = NET_CORE_RMEM_MAX;
             // net/core/rmem_max
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
-        // -- Fred NSC --
-	} else if(strncmp(s, "tcp_rmem", 8) == 0) {
-            name[nlen++] = NET_TCP_RMEM;
-            // net/ipv4/tcp_rmem
-            if(nlen != 3) return 1;
+        } else if(strncmp(s, "rmem_default", 12) == 0) {
+            name[nlen++] = NET_CORE_RMEM_DEFAULT;
+            // net/core/rmem_default
+            if(nlen != 3) return -1;
             break;
-	} else if(strncmp(s, "tcp_wmem", 8) == 0) {
-            name[nlen++] = NET_TCP_WMEM;
-            // net/ipv4/tcp_wmem
-            if(nlen != 3) return 1;
+        } else if(strncmp(s, "wmem_default", 12) == 0) {
+            name[nlen++] = NET_CORE_WMEM_DEFAULT;
+            // net/core/wmem_default
+            if(nlen != 3) return -1;
             break;
-	} else if(strncmp(s, "tcp_mem", 7) == 0) {
-            name[nlen++] = NET_TCP_MEM;
-            // net/ipv4/tcp_mem
-            if(nlen != 3) return 1;
-            break;
-	// --
         } else if(strncmp(s, "tcp_retrans_collapse", 18) == 0) {
             name[nlen++] = NET_IPV4_TCP_RETRANS_COLLAPSE;
             // net/ipv4/tcp_retrans_collapse
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_sack", 8) == 0) {
             name[nlen++] = NET_IPV4_TCP_SACK;
             // net/ipv4/tcp_sack
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_window_scaling", 18) == 0) {
             name[nlen++] = NET_IPV4_TCP_WINDOW_SCALING;
             // net/ipv4/tcp_window_scaling
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_timestamps", 14) == 0) {
             name[nlen++] = NET_IPV4_TCP_TIMESTAMPS;
             // net/ipv4/tcp_timestamps
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_rfc1337", 11) == 0) {
             name[nlen++] = NET_TCP_RFC1337;
             // net/ipv4/tcp_rfc1337
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_stdurg", 10) == 0) {
             name[nlen++] = NET_TCP_STDURG;
             // net/ipv4/tcp_stdurg
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_fack", 8) == 0) {
             name[nlen++] = NET_TCP_FACK;
             // net/ipv4/tcp_fack
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_dsack", 8) == 0) {
             name[nlen++] = NET_TCP_DSACK;
             // net/ipv4/tcp_dsack
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_ecn", 7) == 0) {
             name[nlen++] = NET_TCP_ECN;
             // net/ipv4/tcp_ecn
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_adv_win_scale", 17) == 0) {
             name[nlen++] = NET_TCP_ADV_WIN_SCALE;
             // net/ipv4/tcp_adv_win_scale
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
             break;
         } else if(strncmp(s, "tcp_app_win", 11) == 0) {
             name[nlen++] = NET_TCP_APP_WIN;
             // net/ipv4/tcp_app_win
-            if(nlen != 3) return 1;
+            if(nlen != 3) return -1;
+            break;
+        } else if(strncmp(s, "tcp_rmem", 8) == 0) {
+            name[nlen++] = NET_TCP_RMEM;
+            // net/ipv4/tcp_rmem
+            if(nlen != 3) return -1;
+            break;
+        } else if(strncmp(s, "tcp_wmem", 8) == 0) {
+            name[nlen++] = NET_TCP_WMEM;
+            // net/ipv4/tcp_wmem
+            if(nlen != 3) return -1;
             break;
-/* ------------------------------------------------------------- */
-	} else if(strncmp(s, "tcp_congestion_control", 22) == 0) {
-            name[nlen++] = NET_TCP_CONG_CONTROL;
-            // net/ipv4/tcp_congestion_control
-            if(nlen != 3) return 1;
-	    break;
         }
-/* -- EDIT: Fred for NSC: new functionality in >= 2.6.13 --------*/
 
     } while( (s = strchr(s, '/')) );
 
     if(s == NULL)
-        return 1;
-
+        return -1;
+     
     return do_sysctl(name, nlen, oldval, oldlenp, newval, newlen);
 }
 
@@ -753,7 +749,7 @@ int nsc_setsockopt(void *so, char *optname, void *val, size_t valsize)
 
     if( strcmp(optname, "SO_SNDBUF") == 0 ) opt = SO_SNDBUF;
     else if( strcmp(optname, "SO_RCVBUF") == 0 ) opt = SO_RCVBUF;
-    else return 1;
+    else return -1;
     
     return sock_setsockopt(sock, 0, opt, (char *)val, valsize);
 }
@@ -761,7 +757,7 @@ int nsc_setsockopt(void *so, char *optname, void *val, size_t valsize)
 int nsc_get_tcp_var(void *so, const char *var, char *result, int rlen)
 {
     struct sock *sock = ((struct socket *)so)->sk;
-    struct tcp_sock *tp = tcp_sk(sock);
+    struct tcp_opt *tp = tcp_sk(sock);
 
          if(strcmp(var, "srtt_") == 0)  // in ticks
     {
@@ -799,7 +795,7 @@ int nsc_get_tcp_var(void *so, const char *var, char *result, int rlen)
     }
     else if(strcmp(var, "rxtcur_") == 0)
     {
-        snprintf(result, rlen, "%f", (float)tp->inet_conn.icsk_rto); /* EDIT: Fred for NSC: changed in 2.6.14.2 (inet_conn) */
+        snprintf(result, rlen, "%f", (float)tp->rto);
         return 1;
     }
 
@@ -835,7 +831,8 @@ void nsc_sysctl_r(struct proc_dir_entry *p, const char *d)
         if(table->proc_handler)
         {
             char rbuf[6000];
-            //unsigned int len = 5000;
+            size_t len = 5000;
+            loff_t pos = 0;
             struct file f;
             f.f_pos = 0;
 
@@ -843,7 +840,7 @@ void nsc_sysctl_r(struct proc_dir_entry *p, const char *d)
             memset(rbuf, 0, 5000);
             
             // TODO: make this work for 2.6, below is for 2.4
-            //table->proc_handler(table, 0, &f, rbuf, &len);
+            table->proc_handler(table, 0, &f, rbuf, &len, &pos);
 
             if(strlen(rbuf)) {
                 printk("%s = ", buf2);
@@ -857,3 +854,48 @@ void nsc_sysctl_r(struct proc_dir_entry *p, const char *d)
     }
 }
 
+int nsc_convert_syserr_to_nscerr(int syserr)
+{
+/*
+ * Linux uses negative values for errors, so
+ * we assume >= 0 is a success indicator that doesn't require any mapping.
+ * NSC_E* constants are always < 0.
+ */
+	if (syserr >= 0)
+		return syserr;
+
+	switch (syserr) {
+	case -ERESTART: // fallthrough
+	case -EAGAIN: return NSC_EAGAIN;
+	case -EADDRINUSE: return NSC_EADDRINUSE;
+	case -EADDRNOTAVAIL: return NSC_EADDRNOTAVAIL;
+	case -EALREADY: return NSC_EALREADY;
+	case -ECONNREFUSED: return NSC_ECONNREFUSED;
+	case -ECONNRESET: return NSC_ECONNRESET;
+	case -EHOSTDOWN: return NSC_EHOSTDOWN;
+	case -EHOSTUNREACH: return NSC_EHOSTUNREACH;
+	case -EINPROGRESS: return NSC_EINPROGRESS;
+	case -EISCONN: return NSC_EISCONN;
+	case -EMSGSIZE: return NSC_EMSGSIZE;
+	case -ENETUNREACH: return NSC_ENETUNREACH;
+	case -ENOTCONN: return NSC_ENOTCONN;
+	case -ESHUTDOWN: return NSC_ESHUTDOWN;
+	case -ETIMEDOUT: return NSC_ETIMEDOUT;
+
+	case -ENOTDIR: // used by sysctl and sysctl cradle code
+		printk(KERN_INFO "%s: Saw ENOTDIR\n", __func__);
+		return NSC_ENOTDIR;
+	/* Error conditions that we should never see */
+	case -EPROTONOSUPPORT:
+	case -EDESTADDRREQ:
+	case -EPROTOTYPE:
+	case -ENOBUFS:
+	case -ENONET:
+	case -EPROTO:
+	case -EOVERFLOW: nsc_assert(0, "network stack returned fatal error");
+	}
+
+	printk(KERN_WARNING "%s: Unhandled error number %d\n", __func__, syserr);
+	return NSC_EUNKNOWN;
+}
+
