@@ -111,24 +111,44 @@ private:
                 *sock = tcp;
             }
 
-            return err;
+            return nsc_convert_syserr_to_nscerr(err);
         }
-        
+
+	virtual int getpeername(void *addrspace, size_t *len, int *port)
+	{
+		set_stack_id(parent->stack_id);
+		int err = nsc_getsockpeername(so, addrspace, len, port, 1);
+                return nsc_convert_syserr_to_nscerr(err);
+	}
+
+	virtual int getsockname(void *addrspace, size_t *len, int *port)
+	{
+		set_stack_id(parent->stack_id);
+		int err = nsc_getsockpeername(so, addrspace, len, port, 0);
+                return nsc_convert_syserr_to_nscerr(err);
+	}
+
         virtual int send_data(const void *data, int datalen)
         {
             set_stack_id(parent->stack_id);
-            return nsc_sosend(so, NULL, data, datalen);
+            int err = nsc_sosend(so, NULL, data, datalen);
+	    if (err >= 0)
+		    return err;
+	    err *= -1;
+            return nsc_convert_syserr_to_nscerr(err);
         }
         
-        virtual int read_data(const void *buf, int *buflen)
+        virtual int read_data(void *buf, int *buflen)
         {
             set_stack_id(parent->stack_id);
-            return nsc_soreceive(so, (unsigned char *)buf, buflen);
+            int err = nsc_soreceive(so, (unsigned char *)buf, buflen);
+            return nsc_convert_syserr_to_nscerr(err);
         }
         
         virtual int setsockopt(char *optname, void *val, size_t valsize)
         {
-            return nsc_setsockopt(so, optname, val, valsize);
+            int err = nsc_setsockopt(so, optname, val, valsize);
+            return nsc_convert_syserr_to_nscerr(err);
         }
         
         virtual void print_state(FILE *) 
@@ -289,7 +309,8 @@ public:
         void *newval, size_t newlen)
     {
         set_stack_id(stack_id);
-        return nsc_sysctl(sysctl_name, oldval, oldlenp, newval, newlen);
+        int err = nsc_sysctl(sysctl_name, oldval, oldlenp, newval, newlen);
+        return nsc_convert_syserr_to_nscerr(err);
     }
 
     virtual void buffer_size(int size)
