@@ -20,7 +20,7 @@
 
 */
 
-#define NSC_VERSION 0x000300
+#define NSC_VERSION 0x000400
 
 struct INetStack
 {
@@ -71,6 +71,35 @@ struct INetStack
         return -1;
     }
 
+    // alternate, simpler interface. the stack cradle code is expected
+    // to convert the string-value to something that the stack can handle.
+    // The idea here is that this is a front-end to the sysctl(2) call,
+    // much like the sysctl(8) program.
+    virtual int sysctl_set(const char *name, const char *value)
+    {
+        return -1;
+    }
+
+    // same as above, cradle code is expected to convert the sysctl value
+    // into a string.
+    // returns length of the string in value, i.e. retval > len: 'output truncated'.
+    virtual int sysctl_get(const char *name, char *value, size_t len)
+    {
+        return -1;
+    }
+
+    // this tells the cradle code to put the name of sysctl number 'idx'
+    // into name[].
+    // The idea is that this can be used to get a list of all available sysctls:
+    // char buf[256]
+    // for (i=0; sysctl_getnum(i, buf, sizeof(buf)) > 0 ;i++)
+    //    puts(buf);
+    // returns -1 if idx is out of range and the length of the sysctl name otherwise.
+    virtual int sysctl_getnum(size_t idx, char *name, size_t len)
+    {
+        return -1;
+    }
+
     virtual void show_config()
     {
         ;
@@ -111,7 +140,7 @@ struct INetStreamSocket
     virtual void listen(int) = 0;
     virtual int accept(INetStreamSocket **) = 0;
     virtual int send_data(const void *data, int datalen) = 0;
-    virtual int read_data(const void *buf, int *buflen) = 0;
+    virtual int read_data(void *buf, int *buflen) = 0;
     /* We need to pass the option name in as a string here. The reason for
      * this is that different operating systems you compile on will have
      * different numbers defined for the constants SO_SNDBUF and so on. */
@@ -120,6 +149,23 @@ struct INetStreamSocket
     virtual bool is_connected() = 0;
     virtual bool is_listening() = 0;
 
+    /*
+     * addrspace points to a byte buffer of *addrs_len bytes.
+     * It is filled with a (network order) byte representation of the ip address,
+     * addrs_len is updated to reflect the length (i.e. 4 or eventually 16 for ipv6).
+     * *port is updated with the TCP/SCTP etc. port number.
+     *
+     * The reason that this doesn't use a *struct sockaddr is that the BSDs
+     * have an sa_len member in there, Linux does not, so the struct layout is
+     * different depending on the actual NSC stack and the real host operating
+     * system.
+     */
+    virtual int getpeername(void *addrspace, size_t *addrs_len, int *port) {
+	    return -1;
+    }
+    virtual int getsockname(void *addrspace, size_t *addrs_len, int *port) {
+	    return -1;
+    }
     /* Optional functions used to get and set variables for this TCP
      * connection. */
     virtual bool get_var(const char *var, char *result, int result_len)
